/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  M051 serials ADC demo code

  Show message on Hyper-terminal.
  the baud rate is 115200 bps
  XTAL = 12MHz

  Differential input mode, ADC0 used as Vin+, and ADC1 used as Vin-,
  ADC0 input range is 0~Vref, ADC1 input range is 0~Vref.	the 
  differential iput voltage is equal to (Vadc0 - Vadc1),and the range
  of differential input voltage is -Vref ~ Vref, corresponding to 
  convert data 0 ~ 4095.
  
  Nuvoton Technoledge Corp. PU20 PYLi
  15, Apr, 2010
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//     <q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//  </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
//   <o6.2..3> ADC Clock Source 
//                  <0=> Crystal 12MHz        
//                  <1=> PLL Clock (PLLCON need to select PLL)
//                  <2=> Internal RC 22MHz  
//    <o5.16..23> ADC Clock Divider<1-255><#-1>                                     
//-------- <<< end of configuration section >>> ------------------------------
*/
 
#define CLOCK_SETUP           1
#define CLOCK_EN              0xF
#define PLL_Engine_Enable     1 
#define PLL_SEL               0x00000000 
#define CLOCK_SEL             0x0   
#define ADC_CLOCK_DIVIDER     0x00040000
#define ADC_CLK_Source        0x00000000

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "retarget.h"
#include "UART.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"
#include "Macro_ADC.h"

void TMR0_Delay1ms(uint32_t ulCNT);
void Timer0_Init(void);

/*-----------------------------------------------------------------------------
  Function:		ADC_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	ADC ISR routine.                                    
 *-----------------------------------------------------------------------------*/
void ADC_IRQHandler(void)
{
    uint16_t uiADC1Value;
    /* Clear the A/D interrupt flags for safe */
    set_ADF;  
    uiADC1Value = ADDR0 & 0x00000FFF; 
	printf("adc1 convert value %d\n", uiADC1Value);   
}

/*-----------------------------------------------------------------------------
  Function:		TMR0_Delay1ms                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulCNT:	delay time, unit is millisecond.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer0 used for a delay function.                                    
 *-----------------------------------------------------------------------------*/
void TMR0_Delay1ms(uint32_t ulCNT)
{
    TCMPR0 = 12000;         //Fosc=12MHz, so 12000000/12000=1000Hz=1ms
    setTMR0_CEN;            //Start timer0
    while (ulCNT != 0)
    {
        while ((TISR0&TMR_TIF) != TMR_TIF); //check TIF0
        TISR0 |= TMR_TIF;   //Clear TIF0
        ulCNT --;
    }
    clrTMR0_CEN;            //Stop timer0
}

/*-----------------------------------------------------------------------------
  Function:		Timer0_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer0, period mode.                                    
 *-----------------------------------------------------------------------------*/
void Timer0_Init(void)
{
    TMR0_Clock_EN;
	TMR0ClkSource_ex12MHz;
    TCSR0  = 0x00000000;    //Pre-Scaler
    
    setTMR0_PERIOD;
    setTMR0_IE;             //Timer0 interrupt enable
    setTMR0_CRST;           //Reset the timer/counter0, after set, this bit will be clear by H/W
}

main(void)
{
    Un_Lock_Reg();

    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock

	if(ADC_CLK_Source==0x00000004 )
    {
        PLLCON |= PLL_SEL;
        PLL_Enable();

        /* Wait until PLL is stable. */
		while((CLKSTATUS & PLL_STB) == 0);  
	}   

    Timer0_Init();
    UART_Init(); 
	    
    /* Reset ADC */    
    set_ADC_RST;
    clr_ADC_RST;

	/* ADC engine clock enable */       
    set_ADEN_CLK;
    if (ADC_CLK_Source==0x00000000 )
    {
        ADCClkSource_ex12MHZ;
    }
    else if(ADC_CLK_Source==0x00000004 )
    {
        ADCClkSource_PLL;
    }
    else if(ADC_CLK_Source==0x00000008 )
    {
        ADCClkSource_int22MHZ;
    }

    /* Set ADC divisor */
    CLKDIV=ADC_CLOCK_DIVIDER;   
    /* ADC enable */
    set_ADEN;

    set_CALEN;				//Self Calibration Enable
	while(!(ADCALR&CALDONE));
    
    while(1)
    {
        uint8_t   ucOption;
        
        printf("+----------------------------------------------------------------------+\n");       
        printf("|                          ADC sample code                             |\n");       
       	printf("+----------------------------------------------------------------------+\n");       
        printf("  [1] ADC single mode test(Single input)(Ex:ADC0)\n");
        printf("  [2] ADC single mode test(Differential input)(Ex:ADC0,ADC1)\n");
        printf("  [3] ADC single cycle mode test(Single input)(Ex:ADC0,ADC1)\n");
        printf("  [4] ADC single cycle mode test(Differential input)(Ex:ADC0,ADC1)\n");
        printf("  [5] ADC continuous scan mode test(Single input)(Ex:ADC0,ADC1)\n");
        printf("  [6] ADC continuous scan mode test(Differential input)(Ex:ADC0,ADC1)\n"); 
		printf("  [7] ADC burst mode test(Single input)(Ex:ADC1)\n");     
        ucOption = GetCharPU20();
        printf ("\nkey=%c",ucOption);

        printf("\n");
        switch(ucOption)
        {
            case '1'://[1] ADC single mode test(Single input)
            {
                /* Single mode */ 
                setAD_SIG;              
                            
                /* Single end input */
                clr_DIFFEN;
				/* Set the ADC channel */
                set_CHEN0;
                /* Configure P1.0 as analog input pin */
                set_ADC0_channel;
				/* Disable P1.0 digital input path */
				P1_OFFD |= OFFD0;
                /* Configure P1.0 as input mode */
                P10_InputOnly; 
                /* Clear the A/D interrupt flags for safe */
                set_ADF;       

                while(1)
                {
                    TMR0_Delay1ms(100);
					/* StartADC */
                	set_ADST;  
					while(ADSR&ADF==0);
					set_ADF;
					printf("Single mode(single input) ADC channel0 convert data: %d\n", ADDR0&0xFFF);
                }                               
            }
            case '2'://[2] ADC single mode test(Differential input)
            {
                /* Single mode */ 
                setAD_SIG;              
                            
                /* Differential input */
                set_DIFFEN;
				/* Set the ADC channel */
                set_CHEN0;
                //set_CHEN1;
                /* Configure P1.0,P1.1 as analog input pin */
                set_ADC0_channel;
                set_ADC1_channel;
				/* Disable P1.0, P1.1 digital input path */
				P1_OFFD |= OFFD0;
				P1_OFFD |= OFFD1;
                /* Configure P1.0,P1.1 as input mode */
                P10_InputOnly; 
                P11_InputOnly;
                /* Clear the A/D interrupt flags for safe */
                set_ADF;       

                while(1)
                {
                    TMR0_Delay1ms(100); 
					/* StartADC */
                	set_ADST; 
					while(ADSR&ADF==0);
					set_ADF;
					printf("Single mode(diff input) ADC channel0 convert data: %d\n", ADDR0&0xFFF);
                }
            }
            case '3':
            {
                /* Single-Cycle scan mode */
                setAD_SIGSCAN;              
                            
                /* Single end input */
                clr_DIFFEN;
                /* Set the ADC channel */
                set_CHEN0;
                set_CHEN1;
                /* Configure P1.0,P1.1 as analog input pin */
                set_ADC0_channel;
                set_ADC1_channel;
				/* Disable P1.0, P1.1 digital input path */
				P1_OFFD |= OFFD0;
				P1_OFFD |= OFFD1;
                /* Configure P1.0,P1.1 as input mode */
                P10_InputOnly; 
                P11_InputOnly;
                /* Clear the A/D interrupt flags for safe */
                set_ADF;      

                while(1)
                {
                    TMR0_Delay1ms(100); 
					/* StartADC */
                	set_ADST; 
					while(ADSR&ADF==0);
					set_ADF;
					printf("Single-scan mode(single input) ADC channel0 convert data: %d\n", ADDR0&0xFFF);
					printf("Single-scan mode(single input) ADC channel1 convert data: %d\n\n", ADDR1&0xFFF);
                }
            }
            case '4':
            {
                /* Single-Cycle scan mode */
                setAD_SIGSCAN;                                          
                /* Differential input */
                set_DIFFEN;
                /* Set the ADC channel */
                set_CHEN0;
                //set_CHEN1;
                /* Configure P1.0,P1.1 as analog input pin */
                set_ADC0_channel;
                set_ADC1_channel;
				/* Disable P1.0, P1.1 digital input path */
				P1_OFFD |= OFFD0;
				P1_OFFD |= OFFD1;
                /* Configure P1.0,P1.1 as input mode */
                P10_InputOnly; 
                P11_InputOnly;
                /* Clear the A/D interrupt flags for safe */
                set_ADF;       

                while(1)
                {
                    TMR0_Delay1ms(100);
					/* StartADC */
                	set_ADST;  
					while(ADSR&ADF==0);
					set_ADF;
					printf("Single-scan mode(diff input) ADC channel0 convert data: %d\n", ADDR0&0xFFF);
                }
            }
            case '5':
            {
                /* Continuous scan mode */
                setAD_CONSCAN;              
                            
                /* Single end input */
                clr_DIFFEN;
                /* Set the ADC channel */
                set_CHEN0;
                set_CHEN1;
                /* Configure P1.0,P1.1 as analog input pin */
                set_ADC0_channel;
                set_ADC1_channel;
				/* Disable P1.0, P1.1 digital input path */
				P1_OFFD |= OFFD0;
				P1_OFFD |= OFFD1;
                /* Configure P1.0,P1.1 as input mode */
                P10_InputOnly; 
                P11_InputOnly;
                /* Clear the A/D interrupt flags for safe */
                set_ADF;       
                /* StartADC */  
                set_ADST;
				
                while(1)
				{					
					while(ADSR&ADF==0);
					set_ADF;
					printf("Continuous-scan mode(single input) ADC channel0 convert data: %d\n", ADDR0&0xFFF);
					printf("Continuous-scan mode(single input) ADC channel1 convert data: %d\n\n", ADDR1&0xFFF);
				}
            }
            case '6':
            {
                /* Continuous scan mode */
                setAD_CONSCAN;                                          
                /* Differential input */
                set_DIFFEN;
                /* Set the ADC channel */
                set_CHEN0;
                //set_CHEN1;
                /* Configure P1.0,P1.1 as analog input pin */
                set_ADC0_channel;
                set_ADC1_channel;
				/* Disable P1.0, P1.1 digital input path */
				P1_OFFD |= OFFD0;
				P1_OFFD |= OFFD1;
                /* Configure P1.0,P1.1 as input mode */
                P10_InputOnly; 
                P11_InputOnly;
                /* Clear the A/D interrupt flags for safe */
                set_ADF;      
                /* StartADC */
                set_ADST;

                while(1)
				{
					while(ADSR&ADF==0);
					set_ADF;
					printf("Continuous-scan mode(diff input) ADC channel0 convert data: %d\n", ADDR0&0xFFF);
				}
            } 
			case '7':
            {
                /* Burst mode */ 
                setAD_BURST;              
                            
                /* Single end input */
                clr_DIFFEN;
				/* Set the ADC channel */
                set_CHEN1;
                /* Configure P1.1 as analog input pin */
                set_ADC1_channel;
				/* Disable P1.1 digital input path */
				P1_OFFD |= OFFD1;
                /* Configure P1.1 as input mode */
                P11_InputOnly; 
                /* Clear the A/D interrupt flags for safe */
                set_ADF;    
                /* Enable ADC INT */
                set_ADIE;   
                NVIC_ISER |= ADC_INT;   //NVIC IRQ29 Interrupt Enable    
                /* StartADC */
				set_ADST;

                while(1);                        
            }    
        }
    }
}
