/*---------------------------------------------------------------------*/
/*                                                                     */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.     */
/*                                                                     */
/*---------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  M051 serials I2C demo code

  Microchip I2C EEPROM 24xx256 is used as the slave device.  
  EEPROM is programed by 64 bytes of 00H, 64 bytes of 01H and so on untill 64 
  bytes of 0FH. During Programming, Port 2 will change from 0 to 15, 
  corresponding to EEPROM from page0 to page15.
  Then the program will read programmed data back to verify. If verification
  passes, Port 2 will show 0x5D. If there is any failure occured
  during the progress, Port 2 will show 0xF0.

  Nuvoton Technoledge Corp. PU20 KEHsu
  12, Apr, 2010
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//                  <q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//  </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
// <o5> I2C Clock Rate Selection (22MHz System Clock)
//                  <54=> 100kbps
//                  <13=> 400kbps
//                  <4=> 1Mbps
//-------- <<< end of configuration section >>> ------------------------------
*/

#include <stdint.h>
#include "common.h"
#include "M051.h"
#include "Register_Bit.h"

#define CLOCK_SETUP             0
#define CLOCK_EN                0xF
#define PLL_Engine_Enable       0 
#define PLL_SEL                 0x00080000 
#define CLOCK_SEL               0x0
#define I2C_CLOCK               13

#define EEPROM_SLA              0xa0
#define EEPROM_WR               0x00
#define EEPROM_RD               0x01

#define LED                     P2_DOUT
#define PAGE_SIZE               64

#define ERROR_CODE              0xf0
#define TEST_OK                 0x5d

void Init_I2C(void);
void Write_Page(void);
void Read_Page(void);
void Timed_Write_Cycle(void);

main(void)
{
    Un_Lock_Reg();
    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock
 
    Init_I2C();                                 //Initial I2C0 circuit
    Write_Page();                               //Write Microchip 24xx256 EERPOM
    Read_Page();                                //Verify data

/* ==== Test Pass ==== */   
    LED = TEST_OK;
    while (1);
/* =================== */   
}

/*-----------------------------------------------------------------------------
  Function:		Init_I2C                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize I2C.                                    
 *-----------------------------------------------------------------------------*/
void Init_I2C(void)
{
    P3_PMD &= ~(Px4_PMD | Px5_PMD);
	P3_PMD |= (Px4_OD | Px5_OD);               	//Configure I2C0 pins open-drain

    P3_MFP &= ~(P34_T0_I2CSDA | P35_T1_I2CSCL);
	P3_MFP |= (I2CSDA | I2CSCL);     			//Switch P3.4,P3.5 to I2C0 function pins    
    
	APBCLK |= I2C0_CLKEN;                       //Enable I2C0 clock
    I2CLK = I2C_CLOCK;                        	//Set I2C0 clock rate                                 

    I2CON |= ENSI;                             	//Enable I2C
}

/*-----------------------------------------------------------------------------
  Function:		Write_Page                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Program EEPROM with 64 bytes of 00H, 64 bytes of 01H and so on
  				untill 64 bytes of 0FH.                                    
 *-----------------------------------------------------------------------------*/
void Write_Page(void)
{
    uint8_t ucByte_Count, ucPage, ucProgram_Byte;

                                                //Write Microchip 24xx256 first 16 pages
    for (ucPage = 0; ucPage <= 15; ucPage++)            
    {
                                                //Microchip 24xx256 has 64 bytes per page
        for (ucByte_Count = 0; ucByte_Count <= PAGE_SIZE - 1; ucByte_Count++)       
        {
            // START
            I2CON |= STA;          
            I2CON |= SI;                       	//Clear SI
            while ((I2CON & SI) != SI);	        //Check SI set or not
            I2CON &= ((~STA)&(~SI));           	//STA needs to be cleared after START codition is generated

            if (I2STATUS != 0x08)	            //Check status value after every step
            {
                LED = ERROR_CODE;
                while (1);
            }
    
            //Control byte (SLA+W)  
            I2DAT = EEPROM_SLA | EEPROM_WR;
            I2CON |= SI;
            while ((I2CON & SI) != SI);

            if (I2STATUS != 0x18)              
            {
                LED = ERROR_CODE;
                while (1);
            }
    
            //Address High (DATA)
            I2DAT = ucPage / 4;
            I2CON |= SI; 
            while ((I2CON & SI) != SI);
            if (I2STATUS != 0x28)              
            {
                LED = ERROR_CODE;
                while (1);
            }

            //Address Low (DATA)
            I2DAT = 64* (ucPage % 4) + ucByte_Count;
            I2CON |= SI; 
            while ((I2CON & SI) != SI);
            if (I2STATUS != 0x28)              
            {
                LED = ERROR_CODE;
                while (1);
            }
    
            //Data
            for (ucProgram_Byte = 0; ucProgram_Byte <= (PAGE_SIZE - 1); ucProgram_Byte++)
            {
                I2DAT = ucPage;
                I2CON |= SI; 
                while ((I2CON & SI) != SI);
                if (I2STATUS != 0x28)              
                {
                    LED = ERROR_CODE;
                    while (1);
                }   
            }
    
            //STOP
            I2CON |= STO;
            I2CON |= SI;
            while (I2CON & STO);               	//To ensure STOP condition is generated, polling STO flag
    
            Timed_Write_Cycle();                //After STOP condition, a inner EEPROM timed-write-cycle 
                                                //will occure and EEPROM will not response to outside command
        }

    	P2_DOUT = ucPage;
    }
}

/*-----------------------------------------------------------------------------
  Function:		Timed_Write_Cycle                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	According to requirements of 24Cxx256, when a write operation is 
				complete, ACK polling is needed.                                    
 *-----------------------------------------------------------------------------*/
void Timed_Write_Cycle(void)                    //According to 24Cxx256 datasheet p.9, ACK polling is needed
{
    while (I2STATUS != 0x18)
    {
        //START
        I2CON |= STA;                          	//Check if no ACK is returned by EEPROM, it is under timed-write cycle
        I2CON |= SI; 
        while ((I2CON & SI) != SI);
        I2CON &= ((~STA) & (~SI));

        //Device address
        I2DAT = EEPROM_SLA | EEPROM_WR;
        I2CON |= SI; 
        while ((I2CON & SI) != SI);
    }

    if (I2STATUS != 0x18)                      	//ACK returned. EEPROM can accept next command
        LED = ERROR_CODE;

    //STOP
    I2CON |= STO;
    I2CON |= SI;
    while (I2CON & STO);
}

/*-----------------------------------------------------------------------------
  Function:		Read_Page                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Read out pages have been written and verify them.                                    
 *-----------------------------------------------------------------------------*/
void Read_Page(void)
{
    uint8_t ucPage, ucRead_Byte;

    // START
    I2CON |= STA;          
    I2CON |= SI;                               	//Clear SI
    while ((I2CON & SI) != SI);                	//Check SI set or not
    I2CON &= ((~STA)&(~SI));                   	//STA needs to be cleared after START codition is generated

    if (I2STATUS != 0x08)                      	//Check status value after every step
    {
        LED = ERROR_CODE;
        while (1);
    }

    //Control byte (SLA+W)  
    I2DAT = EEPROM_SLA | EEPROM_WR;
    I2CON |= SI;
    while ((I2CON & SI) != SI);

    if (I2STATUS != 0x18)              
    {
        LED = ERROR_CODE;
        while(1);
    }

    //Address High (DATA)
    I2DAT = 0;
    I2CON |= SI;
    while ((I2CON & SI) != SI);

    if (I2STATUS != 0x28)              
    {
        LED = ERROR_CODE;
        while (1);
    }

    //Address Low (DATA)
    I2DAT = 0;
    I2CON |= SI; 
    while ((I2CON & SI) != SI);
    if (I2STATUS != 0x28)              
    {
        LED = ERROR_CODE;
        while (1);
    }

    // Repeated START
    I2CON |= STA;          
    I2CON |= SI;                               	//Clear SI
    while ((I2CON & SI) != SI);                	//Check SI set or not
    I2CON &= ((~STA)&(~SI));                   	//STA needs to be cleared after START codition is generated

    if (I2STATUS != 0x10)                      	//Check status value after every step
    {
        LED = ERROR_CODE;
        while (1);
    }

    //Control byte (SLA+R)  
    I2DAT = EEPROM_SLA | EEPROM_RD;
    I2CON |= SI;
    while ((I2CON & SI) != SI);

    if (I2STATUS != 0x40)              
    {
        LED = ERROR_CODE;
        while (1);
    }

    I2CON |= AA;

    for (ucPage = 0; ucPage <= 15; ucPage++)    //Read Microchip 24xx256 first 16 pages
    {
        //Data
        for (ucRead_Byte = 0; ucRead_Byte <= (PAGE_SIZE - 1); ucRead_Byte++)
        {
            I2CON |= SI; 
            while ((I2CON & SI) != SI);
            if (I2STATUS != 0x50)              
            {
                LED = ERROR_CODE;
                while (1);
            }

    
            if (I2DAT != (ucPage))             	//Verify 
            {
                LED = ERROR_CODE;
                while (1);
            }
        }
    }   

    //send a NACK to disconnect 24xx256
    I2CON &= (~AA);
    I2CON |= SI; 
    while ((I2CON & SI) != SI);
        
    //STOP
    I2CON |= STO;
    I2CON |= SI;
    while (I2CON & STO);                       	//To ensure STOP condition is generated, polling STO flag
}
