/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"
#include "Retarget.h"
#include "Uart.h"
#include "Common.h"
#include "L_SPI.h"

extern uint32_t ucgSPI0_Divider_Value;

/*-----------------------------------------------------------------------------
  Function:		SPI0_Master_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize SPI0 as master.                                    
 *-----------------------------------------------------------------------------*/
void SPI0_Master_Init(void)
{                               
	P1_MFP &= ~(P14_AIN4_SPI0SS |
				P15_AIN5_SPI0MOSI |
				P16_AIN6_SPI0MISO |
				P17_AIN7_SPI0CLK) ;  
	
	P1_MFP |= (SPI0SS |
			   SPI0MOSI |
			   SPI0MISO |
			   SPI0CLK) ;   	//Enable SPI0 Pin in I/O

	APBCLK |= SPI0_CLKEN;       //SPI0 Clock Enable

    							//             ___          ____ 
    SPI0_SSR &= ~LVL_H;   		//SS_LVL = 0     l_________l

    SPI0_CNTRL &= ~LSB_FIRST;   //LSB  = 0, MSB Transmit/Receiv first.
    SPI0_CNTRL &= ~CLKP_IDLE_H; //CLKP = 0, SCLK Idel Low
    SPI0_CNTRL |= TX_NEG_F;     //Tx_NEG = 1, The SDO signal is changed on the falling edge.
    SPI0_CNTRL &= ~RX_NEG_F;    //Rx_NEG = 0, The SDI signal is latched on the rising edge.

    CLKDIV &= 0xFFFFFFF0;       //HCLK_N = 0, Pclk = SYSclk/(HCLK_N+1)
    SPI0_DIVIDER &= 0xFFFF0000; //DIVIDER = 0x0000, SPIclk = Pclk/((HCLK_N+1)*2)
    SPI0_DIVIDER |= ucgSPI0_Divider_Value;

    SPI0_CNTRL &= ~SPI_MODE_SLAVE;    //SPI = Master Mode
    SPI0_SSR &= ~ASS_AUTO;       	  //Slave Select non-auto
}

/*-----------------------------------------------------------------------------
  Function:		SPI0_Send_1W                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulData: data to be sent
				ucLength: SPI0 transmit bit length, range is 0x00~0x20                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	SPI0 send a data                                    
 *-----------------------------------------------------------------------------*/
void SPI0_Send_1W(uint32_t ulData, uint8_t ucLength)
{
    SPI0_CNTRL &= TX_NUM_ONE;   //Only one Tx0/Rx0
    SPI0_Length(ucLength);
    SPI0_TX0 = ulData;
    SPI0_CNTRL |= GO_BUSY;      //Start Transfer
}

/*-----------------------------------------------------------------------------
  Function:		Get_SPI_Flash_ID                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Get used SPI flash ID, and print via UART0.                                    
 *-----------------------------------------------------------------------------*/
void Get_SPI_Flash_ID(void)
{
    uint32_t ulDevice_ID, ulManufacturer_ID;

	SPI0_SSR |= SSR_ACT;				  //Chip select active
    SPI0_Send_1W(0x90000000, 0x20);
    while((SPI0_CNTRL & GO_BUSY)!=0);
    SPI0_Send_1W(0xFFFF, 0x10);			  //When read from SPI device, send some dummy bits	first.
    while((SPI0_CNTRL & GO_BUSY)!=0);
	SPI0_SSR &= ~SSR_ACT;				  //Chip select inactive

    ulDevice_ID = SPI0_RX0 & 0x000000FF;
    ulManufacturer_ID = SPI0_RX0 & 0x0000FF00;
    ulManufacturer_ID >>= 8;
    
    printf("Manufacturer ID = 0x%X\n", ulManufacturer_ID);
    printf("Device ID = 0x%X\n", ulDevice_ID);
}

/*-----------------------------------------------------------------------------
  Function:		Program_Page0_Data                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Program SPI flash page0 with data 0x00,0x01, ... 0xFF.                                    
 *-----------------------------------------------------------------------------*/
void Program_Page0_Data(void)
{
    uint16_t uiCounter1 = 0;
    //Write Enable
	SPI0_SSR |= SSR_ACT;
    SPI0_Send_1W(WRITER_ENABLE, 0x08);
    while((SPI0_CNTRL & GO_BUSY)!=0);
	SPI0_SSR &= ~SSR_ACT;
        
    //Page Program Data
    printf("Page0 Program Data, Data = 0x00,0x01,0x02....0xFF\n");
	SPI0_SSR |= SSR_ACT;
    SPI0_Send_1W(PAGE_PROGRAM, 0x08);                  //Write Instruction
    while((SPI0_CNTRL & GO_BUSY)!=0);
    SPI0_Send_1W(0x000000, 24);                        //Write Address
    while((SPI0_CNTRL & GO_BUSY)!=0);
    for(uiCounter1=0; uiCounter1 < 256;uiCounter1++)   //Write Data
    {
        SPI0_Send_1W(uiCounter1, 8);
        while((SPI0_CNTRL & GO_BUSY)!=0);
    }
    SPI0_SSR &= ~SSR_ACT;
	    
    //Write Disable
	SPI0_SSR |= SSR_ACT;
    SPI0_Send_1W(WRITER_DISABLE, 0x08);
    while((SPI0_CNTRL & GO_BUSY)!=0);
	SPI0_SSR &= ~SSR_ACT;
}

/*-----------------------------------------------------------------------------
  Function:		Read_Page0_Data                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Read SPI flash page0, print in Hyper-terminal via UART0                                    
 *-----------------------------------------------------------------------------*/
void Read_Page0_Data(void)
{
    uint16_t uiCounter1 = 0;

    printf("Page0 Read Data, Data =\n");

	SPI0_SSR |= SSR_ACT;
    SPI0_Send_1W(READ_DATAE, 0x08);                    //Read Instruction
    while((SPI0_CNTRL & GO_BUSY)!=0);
    SPI0_Send_1W(0x000000, 24);                        //Write Address
    while((SPI0_CNTRL & GO_BUSY)!=0);
    for(uiCounter1=0; uiCounter1 < 256;uiCounter1++)   //Read Data
    {                                                  
        SPI0_Send_1W(0xFF, 8);   					   //Must be Send 0xFF Data
        while((SPI0_CNTRL & GO_BUSY)!=0);
        printf("%2x,",(SPI0_RX0&0x000000FF));
        if((uiCounter1%0x10) == 0x0F)
        printf("\n");
    }
	SPI0_SSR &= ~SSR_ACT;
}

/*-----------------------------------------------------------------------------
  Function:		Erase_All_Chip                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Erase all pages of SPI flash.                                    
 *-----------------------------------------------------------------------------*/
void Erase_All_Chip(void)
{
    //Write Enable
	SPI0_SSR |= SSR_ACT;
    SPI0_Send_1W(WRITER_ENABLE, 0x08);
    while((SPI0_CNTRL & GO_BUSY)!=0);
	SPI0_SSR &= ~SSR_ACT;
    
    //Chip Erase
    printf("Chip Erase\n");
	SPI0_SSR |= SSR_ACT;
    SPI0_Send_1W(CHIP_ERASE, 0x08);                    //Erase Instruction
    while((SPI0_CNTRL & GO_BUSY)!=0);
	SPI0_SSR &= ~SSR_ACT;

    //Write Disable
	SPI0_SSR |= SSR_ACT;
    SPI0_Send_1W(WRITER_DISABLE, 0x08);
    while((SPI0_CNTRL & GO_BUSY)!=0);
	SPI0_SSR &= ~SSR_ACT;
}

/*-----------------------------------------------------------------------------
  Function:		SPI0_Length                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ucLength: SPI0 transmit bit length, range is 0x00~0x20                                      
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Set SPI0 transmit data length.                                    
 *-----------------------------------------------------------------------------*/
void SPI0_Length(uint8_t ucLength) 
{
    if(ucLength <= 0x20)
    {
        if((ucLength & 0x01) == 0)
            SPI0_CNTRL &= ~(1<<3);
        else
            SPI0_CNTRL |= (1<<3);
        if((ucLength & 0x02) == 0)
            SPI0_CNTRL &= ~(1<<4);
        else
            SPI0_CNTRL |= (1<<4);

        if((ucLength & 0x04) == 0)
            SPI0_CNTRL &= ~(1<<5);
        else
            SPI0_CNTRL |= (1<<5);

        if((ucLength & 0x08) == 0)
            SPI0_CNTRL &= ~(1<<6);
        else
            SPI0_CNTRL |= (1<<6);

        if((ucLength & 0x10) == 0)
            SPI0_CNTRL &= ~(1<<7);
        else
            SPI0_CNTRL |= (1<<7);
    } // end if(ucLength < 0x20)
}
