/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
	This demo is used for testing timer0~timer3, in period and one-shot mode, 
	using external 12M clock.
	
	Run the demo, select a timer(timer0~timer3) follow wizard, then select timer 
	mode(period and one-shot) in the same way, now you can see the result print 
	through Hyper-terminal.
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz       
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           1
#define CLOCK_EN              0x1F
#define PLL_Engine_Enable     0 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x0

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "UART.h"
#include "Retarget.h"

static uint32_t TMR_Tick = 0;
static uint32_t TMR_Shot = 0;
static uint8_t TMR_Select_Flag = 0;
static uint8_t TMR_Mode_Select_Flag = 0;

static void TMR_Select(void);
static void TMR_Mode_Select(void);

void TMR0_Init(void);
void TMR1_Init(void);
void TMR2_Init(void);
void TMR3_Init(void);

int main(void)
{
	/* Unlock registers had been proctected */
	Un_Lock_Reg();

	/* Enable external 12M Crystal */
	PWRCON |= XTL12M_EN;
	/* Wait until 12M clock is stable. */
	while((CLKSTATUS & XTL12M_STB) == 0); 	

	/* HCLK clock source select 12M external Crystal */
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;

	/* Lock procteted registers */
	Lock_Reg();			

	/* Initialize UART */
	UART_Init();

	while(1)
    {
		TMR_Tick = 0;
		TMR_Shot = 0;
        TMR_Select();
        TMR_Select_Flag = getchar();
        printf("%c\n",TMR_Select_Flag);
        switch(TMR_Select_Flag)
        {
            case '1':    
				TMR0_Init();
				break;

            case '2':    
            	TMR1_Init();
				break;
            case '3':    
            	TMR2_Init();
				break;
            
            case '4':    
				TMR3_Init();
				break;
           
            default:
                break; 
        }

		switch(TMR_Mode_Select_Flag)
        {
            case '1':    
				while(TMR_Tick != 100);
				break;

            case '2':    
            	while(TMR_Shot != 1);
				break;
           
            default:
                break; 
        }
    }
}

/*-----------------------------------------------------------------------------
  Function:		TMR_Select                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer select.                                    
 *-----------------------------------------------------------------------------*/
static void TMR_Select (void)
{
    printf("\n\n");
    printf("Please select a timer for test...\n");
    printf("+-----------------------------------------------------------+\n");
    printf("| Timer0                                      	      - [1] |\n");
    printf("| Timer1                             	              - [2] |\n");
    printf("| Timer2                             	              - [3] |\n");
    printf("| Timer3                             	              - [4] |\n");
    printf("+-----------------------------------------------------------+\n");
    printf("Select key : ");
}

/*-----------------------------------------------------------------------------
  Function:		TMR_Mode_Select                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer mode select.                                    
 *-----------------------------------------------------------------------------*/
static void TMR_Mode_Select(void)
{
	printf("\n\n");
    printf("Please select timer mode...\n");
    printf("+-----------------------------------------------------------+\n");
    printf("| 10ms period mode                             	      - [1] |\n");
    printf("| 1s one-shot mode                             	      - [2] |\n");
    printf("+-----------------------------------------------------------+\n");
    printf("Select key : ");	
}

/*-----------------------------------------------------------------------------
  Function:		TMR0_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer0.                                    
 *-----------------------------------------------------------------------------*/
void TMR0_Init(void)
{
	/* Enable Timer0 clock source */
    APBCLK |= TMR0_CLKEN;
	/* Select Timer0 clock source as external 12M */  
	CLKSEL1 = (CLKSEL1 & (~TM0_CLK)) | TM0_12M;     	

	/* Reset IP TMR0 */
	IPRSTC2 |= TMR0_RST;
	IPRSTC2 &= ~TMR0_RST;

	TMR_Mode_Select();
    TMR_Mode_Select_Flag = getchar();
    printf("%c\n",TMR_Mode_Select_Flag);
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			/* Select timer0 Operation mode as period mode */	
			TCSR0 &= ~TMR_MODE;
			TCSR0 |= MODE_PERIOD;			

			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR0 = TCSR0 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR0 = 120000;				// Set TCMPR [0~16777215]
			break;

		case '2':
			/* Select timer0 Operation mode as one-shot mode */	
			TCSR0 &= ~TMR_MODE;
			TCSR0 |= MODE_ONE_SHOT;		
			
			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR0 = TCSR0 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR0 = 12000000;				// Set TCMPR [0~16777215]
			break;	
	}

	/* Enable timer0 interrupt */
	TCSR0 |= TMR_IE;		
	NVIC_ISER |= TMR0_INT;	

	/* Reset timer0 counter */
	TCSR0 |= CRST;	

	/* Enable Timer0 */					
	TCSR0 |= CEN;						
}

/*-----------------------------------------------------------------------------
  Function:		TMR0_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer0 ISR routine                                    
 *-----------------------------------------------------------------------------*/
void TMR0_IRQHandler(void)
{
	/* Clear timer0 interrupt flag */
	TISR0 |= TMR_TIF;
	 			
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			TMR_Tick++;
			printf("Timer0 10ms tick %d\n", TMR_Tick);
			if (TMR_Tick == 100)
			{
				TCSR0 &= ~CEN;
				printf("\nTimer0 10ms period test is over\n");
			}
			break;

		case '2':
			printf("\nTimer0 1s one-shot test is over\n");
			TMR_Shot = 1;
			break;
	}
}

/*-----------------------------------------------------------------------------
  Function:		TMR1_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer1.                                    
 *-----------------------------------------------------------------------------*/
void TMR1_Init(void)
{
	/* Enable Timer0 clock source */
    APBCLK |= TMR1_CLKEN;
	/* Select Timer0 clock source as external 12M */  
	CLKSEL1 = (CLKSEL1 & (~TM1_CLK)) | TM1_12M; 	

	/* Reset IP TMR1 */
	IPRSTC2 |= TMR1_RST;
	IPRSTC2 &= ~TMR1_RST;

	TMR_Mode_Select();
    TMR_Mode_Select_Flag = getchar();
    printf("%c\n",TMR_Mode_Select_Flag);
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			/* Select timer1 Operation mode as period mode */	
			TCSR1 &= ~TMR_MODE;
			TCSR1 |= MODE_PERIOD;		

			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR1 = TCSR1 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR1 = 120000;				// Set TCMPR [0~16777215]
			break;

		case '2':
			/* Select timer1 Operation mode as one-shot mode */	
			TCSR1 &= ~TMR_MODE;
			TCSR1 |= MODE_ONE_SHOT;	
			
			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR1 = TCSR1 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR1 = 12000000;				// Set TCMPR [0~16777215]
			break;	
	}

	/* Enable timer1 interrupt */
	TCSR1 |= TMR_IE;		
	NVIC_ISER |= TMR1_INT;	

	/* Reset timer1 counter */
	TCSR1 |= CRST;	

	/* Enable Timer1 */					
	TCSR1 |= CEN;			
}

/*-----------------------------------------------------------------------------
  Function:		TMR1_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer1 ISR routine                                    
 *-----------------------------------------------------------------------------*/
void TMR1_IRQHandler(void)
{
	/* Clear timer1 interrupt flag */
	TISR1 |= TMR_TIF; 	
			
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			TMR_Tick++;
			printf("Timer1 10ms tick %d\n", TMR_Tick);
			if (TMR_Tick == 100)
			{
				TCSR1 &= ~CEN;
				printf("\nTimer1 10ms period test is over\n");
			}
			break;

		case '2':
			printf("\nTimer1 1s one-shot test is over\n");
			TMR_Shot = 1;
			break;
	}
}

/*-----------------------------------------------------------------------------
  Function:		TMR2_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer2.                                    
 *-----------------------------------------------------------------------------*/
void TMR2_Init(void)
{
	/* Enable Timer0 clock source */
    APBCLK |= TMR2_CLKEN;
	/* Select Timer0 clock source as external 12M */  
	CLKSEL1 = (CLKSEL1 & (~TM2_CLK)) | TM2_12M;	

	/* Reset IP TMR2 */
	IPRSTC2 |= TMR2_RST;
	IPRSTC2 &= ~TMR2_RST;

	TMR_Mode_Select();
    TMR_Mode_Select_Flag = getchar();
    printf("%c\n",TMR_Mode_Select_Flag);
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			/* Select timer2 Operation mode as period mode */	
			TCSR2 &= ~TMR_MODE;
			TCSR2 |= MODE_PERIOD;			

			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR2 = TCSR2 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR2 = 120000;				// Set TCMPR [0~16777215]
			break;

		case '2':
			/* Select timer2 Operation mode as one-shot mode*/	
			TCSR2 &= ~TMR_MODE;
			TCSR2 |= MODE_ONE_SHOT;		
			
			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR2 = TCSR2 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR2 = 12000000;				// Set TCMPR [0~16777215]
			break;	
	}

	/* Enable timer2 interrupt */
	TCSR2 |= TMR_IE;		
	NVIC_ISER |= TMR2_INT;	

	/* Reset timer2 counter */
	TCSR2 |= CRST;	

	/* Enable Timer2 */					
	TCSR2 |= CEN;			
}

/*-----------------------------------------------------------------------------
  Function:		TMR2_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer2 ISR routine                                    
 *-----------------------------------------------------------------------------*/
void TMR2_IRQHandler(void)
{
	/* Clear timer2 interrupt flag */
	TISR2 |= TMR_TIF; 	
			
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			TMR_Tick++;
			printf("Timer2 10ms tick %d\n", TMR_Tick);
			if (TMR_Tick == 100)
			{
				TCSR2 &= ~CEN;
				printf("\nTimer2 10ms period test is over\n");
			}
			break;

		case '2':
			printf("\nTimer2 1s one-shot test is over\n");
			TMR_Shot = 1;
			break;
	}
}

/*-----------------------------------------------------------------------------
  Function:		TMR3_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer3.                                    
 *-----------------------------------------------------------------------------*/
void TMR3_Init(void)
{
	/* Enable Timer0 clock source */
    APBCLK |= TMR3_CLKEN;
	/* Select Timer0 clock source as external 12M */  
	CLKSEL1 = (CLKSEL1 & (~TM3_CLK)) | TM3_12M;	

	/* Reset IP TMR3 */
	IPRSTC2 |= TMR3_RST;
	IPRSTC2 &= ~TMR3_RST;

	TMR_Mode_Select();
    TMR_Mode_Select_Flag = getchar();
    printf("%c\n",TMR_Mode_Select_Flag);
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			/* Select timer3 Operation mode as period mode */	
			TCSR3 &= ~TMR_MODE;
			TCSR3 |= MODE_PERIOD;		

			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR3 = TCSR3 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR3 = 120000;				// Set TCMPR [0~16777215]
			break;

		case '2':
			/* Select timer3 Operation mode as one-shot mode */	
			TCSR3 &= ~TMR_MODE;
			TCSR3 |= MODE_ONE_SHOT;		
			
			/* Select Time out period = (Period of timer clock input) * (8-bit Prescale + 1) * (24-bit TCMP)*/
			TCSR3 = TCSR3 & 0xFFFFFF00;		// Set Prescale [0~255]
			TCMPR3 = 12000000;				// Set TCMPR [0~16777215]
			break;	
	}

	/* Enable timer3 interrupt */
	TCSR3 |= TMR_IE;		
	NVIC_ISER |= TMR3_INT;	

	/* Reset timer3 counter */
	TCSR3 |= CRST;	

	/* Enable Timer3 */					
	TCSR3 |= CEN;		
}

/*-----------------------------------------------------------------------------
  Function:		TMR3_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer3 ISR routine                                    
 *-----------------------------------------------------------------------------*/
void TMR3_IRQHandler(void)
{
	/* Clear timer3 interrupt flag */
	TISR3 |= TMR_TIF;
	 			
	switch(TMR_Mode_Select_Flag)
	{
		case '1':
			TMR_Tick++;
			printf("Timer3 10ms tick %d\n", TMR_Tick);
			if (TMR_Tick == 100)
			{
				TCSR3 &= ~CEN;
				printf("\nTimer3 10ms period test is over\n");
			}
			break;

		case '2':
			printf("\nTimer3 1s one-shot test is over\n");
			TMR_Shot = 1;
			break;
	}
}
