/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  This demo is used for testing watchdog of M051 serial MCU, main function is 
  to test watch dog timer 8 timeout interval by turn.
  Run the demo, the result will be shown on the Hyper-terminal.
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           1
#define CLOCK_EN              0x1F
#define PLL_Engine_Enable     0 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x0

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "UART.h"
#include "Retarget.h"

volatile uint32_t WDT_Flag = 0;

void WDT_Init(void);
void WDT_SetInterval(uint32_t interval);

int main(void)
{
	/* Unlock registers had been proctected */
	Un_Lock_Reg();

	/* Enable external 12M Crystal */
	PWRCON |= XTL12M_EN;
	/* Wait until 12M clock is stable. */
	while((CLKSTATUS & XTL12M_STB) == 0);

	/* HCLK clock source select 12M external Crystal */
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	

	/* Lock procteted registers */
	Lock_Reg();			

	/* Initialize UART */
	UART_Init();

	printf("Press Enter key to start\n");
	getchar();

	/* Initialize WDT */
	WDT_Init();	

	printf("First interval test...\n");
	Un_Lock_Reg();
	/* Set WDT timeout Interval	as 69.33us */
	WDT_SetInterval(0);	
	/* Reset WDT timer count value */
	WTCR |= CLRWTR;
	Lock_Reg();

	while(1)
	{
		if (WDT_Flag == 8)		 		/* Test is over, Disable WDT */
		{
			Un_Lock_Reg();
			/* Disable WDT */
			WTCR &= ~WTE;
			Lock_Reg();
			break;
		}
	}
	
	printf("WDT test is over\n");

	return 0;
}

/*-----------------------------------------------------------------------------
  Function:		WDT_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	WDT ISR routine                                    
 *-----------------------------------------------------------------------------*/
void WDT_IRQHandler(void) 
{ 
	Un_Lock_Reg(); 
	/* Clear WDT interrupt flag */
	WTCR |= WTIF;

	WDT_Flag++;
	switch(WDT_Flag)					/* Test WDT each timeout Interval by turn */
	{
		case 1:
			printf("Second interval test...\n");
			/* Set WDT timeout Interval	as 72.53us */
			WDT_SetInterval(1);
			WTCR |= CLRWTR;
			break;

		case 2:
			printf("Third interval test...\n");
			/* Set WDT timeout Interval	as 85.33us */
			WDT_SetInterval(2);
			WTCR |= CLRWTR;
			break;

		case 3:
			printf("Fourth interval test...\n");
			/* Set WDT timeout Interval	as 170.67us */
			WDT_SetInterval(3);
			WTCR |= CLRWTR;
			break;

		case 4:
			printf("Fifth interval test...\n");
			/* Set WDT timeout Interval	as 426.67us */
			WDT_SetInterval(4);
			WTCR |= CLRWTR;
			break;

		case 5:
			printf("Sixth interval test...\n");
			/* Set WDT timeout Interval	as 1.45ms */
			WDT_SetInterval(5);
			WTCR |= CLRWTR;
			break;

		case 6:
			printf("Seventh interval test...\n");
			/* Set WDT timeout Interval	as 5.55ms */
			WDT_SetInterval(6);
			WTCR |= CLRWTR;
			break;

		case 7:
			printf("Eighth interval test...\n");
			/* Set WDT timeout Interval	as 21.93ms */
			WDT_SetInterval(7);
			WTCR |= CLRWTR;
			break;

		default:			
			break;			
	}

	Lock_Reg();
}

/*-----------------------------------------------------------------------------
  Function:		WDT_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize WDT, select external 32K Crystal as WDT clock source.                                    
 *-----------------------------------------------------------------------------*/
void WDT_Init(void)
{
	Un_Lock_Reg();	
		
	/* Enable WDT clock */  
	APBCLK |= WDT_CLKEN;

	/* Set WDT clock source as 12M */
	CLKSEL1 = (CLKSEL1 & (~WDT_CLK)) | WDT_12M;
				
	/* Disable Watchdog Timer Reset function */
	WTCR &= WTRE;	

	/* Set WDT timeout Interval	as 21.93ms */
	WDT_SetInterval(7);

	/* Enable WDT interrupt */			
	WTCR |= WTIE;
	NVIC_ISER |= WDT_INT;

	/* Enable WDT */		
	WTCR |= WTE;
		
	Lock_Reg();	 
}

/*-----------------------------------------------------------------------------
  Function:		WDT_SetInterval                                                                         
                                                                                                         
  Parameter:        																					   	
	        	interval: WDT overflow interval, range is 0~7.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Select WDT Timeout interval                                    
 *-----------------------------------------------------------------------------*/
void WDT_SetInterval(uint32_t interval)
{
	/* Select Timeout Interval */
	WTCR = (WTCR&(~WTIS)) | (interval<<8);
}
