/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  Main function of this demo is wakeup M051 serial MCU from Power down mode by 
  watch dog, please pay attention that after download the demo code into MCU, 
  power off and power on chip, then start to test.
  Run the demo, relative information, including MCU enter and wakeup from power 
  down mode, will be shown via Hyper-terminal. 
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           0
#define CLOCK_EN              0x1F
#define PLL_Engine_Enable     0 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x0

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "UART.h"
#include "Retarget.h"

void WDT_Init(void);
void WDT_SetInterval(uint32_t interval);

void delay(void);

int main(void)
{
	/* Unlock registers had been proctected */
	Un_Lock_Reg();
		
	/* Enable external 12M Crystal */
	PWRCON |= XTL12M_EN;
	/* Wait until 12M clock is stable. */
	while((CLKSTATUS & XTL12M_STB) == 0);

	/* Enable internal 10k Osc */
	PWRCON |= OSC10K_EN;
	/* Wait until 10K clock is stable. */
	while((CLKSTATUS & OSC10K_STB) == 0);

	/* HCLK clock source select 12M external Crystal */
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;

	/* Lock procteted registers */
	Lock_Reg();			

	/* Initialize UART */
	UART_Init();

	printf("Press Enter key to start...\n");
	getchar();

	/* Initialize WDT */
	WDT_Init();	

	Un_Lock_Reg();
	/* Select Timeout Interval 1740.8ms*/
	WDT_SetInterval(5);	
	/* Reset WDT timer */
	WTCR |= CLRWTR;
	Lock_Reg();

	while(1)
	{
		printf("Enter power down mode...\n");
		delay();

		/* Enter power down mode */
		Un_Lock_Reg();
		/* Indicate waking from sleep might take longer. */
		SCR |= SLEEPDEEP;
		/* Disable interrupt occur when Power down mode wakeup */
		PWRCON &= ~PD_WU_IE;
		/* Power down control */
		PWRCON &= ~PD_WAIT_CPU;
		PWRCON |= PWR_DOWN_EN;
		Lock_Reg();
	
		delay();
		printf("WakeUp from power down mode...\n\n");
	}
}

/*-----------------------------------------------------------------------------
  Function:		WDT_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	WDT ISR routine                                    
 *-----------------------------------------------------------------------------*/
void WDT_IRQHandler(void) 
{ 
	Un_Lock_Reg(); 
	WTCR |= WTWKF;
	WTCR |= WTIF;
	Lock_Reg();
	printf("WDT interrupt!\n");
}

/*-----------------------------------------------------------------------------
  Function:		WDT_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize WDT, select external 32K Crystal as WDT clock source.                                    
 *-----------------------------------------------------------------------------*/
void WDT_Init(void)
{
	Un_Lock_Reg();	
		
	/* Enable WDT clock */ 
	APBCLK |= WDT_CLKEN;

	/* Set WDT clock source as 10k */
	CLKSEL1 = (CLKSEL1 & (~WDT_CLK)) | WDT_10K;
				
	/* Disable Watchdog Timer Reset function */
	WTCR &= ~WTRE;	

	/* Set WDT timeout Interval	as 101.44ms */
	WDT_SetInterval(0);

	/* Enable WDT interrupt */			
	WTCR |= WTIE;
	NVIC_ISER |= WDT_INT;

	/* Enable WDT wakeup function */
	WTCR |= WTWKE;

	/* Enable WDT */
	WTCR |= WTE;
		
	Lock_Reg();	 
}

/*-----------------------------------------------------------------------------
  Function:		WDT_SetInterval                                                                         
                                                                                                         
  Parameter:        																					   	
	        	interval: WDT overflow interval, range is 0~7.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Select WDT Timeout interval                                    
 *-----------------------------------------------------------------------------*/
void WDT_SetInterval(uint32_t interval)
{
	/* Select Timeout Interval */
	WTCR = (WTCR&(~WTIS)) | (interval<<8);
}

void delay(void)
{
	uint32_t i;

	for (i=0; i<10000; i++);
}
